package com.biganto.visual.roompark.conductor.dialogs.tour_chooser

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.ProgressBar
import android.widget.TextView
import androidx.annotation.LayoutRes
import androidx.core.os.bundleOf
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.interactor.ToursInteractor
import com.biganto.visual.roompark.domain.model.TourModel
import com.biganto.visual.roompark.domain.use_case.DownloadUseCase
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreviewEntity
import com.bluelinelabs.conductor.Controller
import com.bumptech.glide.Glide
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import jp.wasabeef.glide.transformations.BlurTransformation
import jp.wasabeef.glide.transformations.ColorFilterTransformation
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

internal const val  DOWNLOAD_TOUR_ID_KEY = "DOWNLOAD_TOUR_ID_KEY"


class DownloadTourDialogController : Controller {

    constructor(args: Bundle) : super(args)

    constructor(tourArg: TourModel) : super(bundleOf(DOWNLOAD_TOUR_ID_KEY to tourArg))

    lateinit var progressBarDownload: ProgressBar
    lateinit var downloaderBg: ImageView
    lateinit var downloadTourTitleText: TextView
    lateinit var cancelDownloadText: TextView

    private val tour:TourModel by lazy {
         args.getParcelable<TourModel>(DOWNLOAD_TOUR_ID_KEY)
    }

    private var downloadToken = DownloadUseCase.CancellationToken(false)

    @Inject
    lateinit var downloader: ToursInteractor

    private val disposables = CompositeDisposable()

    override fun onDetach(view: View) {
        disposables.clear()
        super.onDetach(view)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        progressBarDownload = view.findViewById(R.id.downloadProgress)
        downloaderBg = view.findViewById(R.id.backgroundDownloader)
        cancelDownloadText = view.findViewById(R.id.cancelDownloadButton)
        downloadTourTitleText = view.findViewById(R.id.tourToDownloadTitle)

//        progress.visibility = View.VISIBLE

        downloadToken.isCancelled = true
        downloadToken = DownloadUseCase.CancellationToken(false)

        downloadTourTitleText.text = tour.title
        Glide.with(view)
            .load(tour.previewUrl)
            .transform(BlurTransformation(25, 1))
            .transform(ColorFilterTransformation(0x99000000.toInt()))
            .into(downloaderBg)

        disposables.add(
            downloader.downloadTour(tour.tour_id, downloadToken)
                .onErrorReturn { Timber.e(it);TourPreviewEntity() }
//                .onErrorReturn { parseError(it);TourPreviewEntity() }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe (
                    { model ->
                        updateProgressBar(model.downloadedFiles, model.overallFiles)
                        if (model.overallFiles == model.downloadedFiles)
                        {
                            Timber.w("FINISH DOIWLNLOD")
                            //Start PLayer
                        }
//                            activity?.runOnUiThread { startPlayerById(model.id) }
                    }
                    ,{error ->
                        Timber.e(error)
//                        snackbars.showSnackBar("Ошибка во время загрузки тура!")
                    }
                ))
//        downloadTour(it.tour.tour_id, downloadToken)

        view.findViewById<View>(R.id.close_current_button).setOnClickListener { handleBack() }

        return view
    }



    private fun updateProgressBar(curr:Int, max:Int){
        activity?.runOnUiThread {
            progressBarDownload.max = max
            progressBarDownload.progress = curr
        }
    }

    @LayoutRes
    fun getLayoutId() = R.layout.photo_viewer

    override fun handleBack(): Boolean {
        return router.popCurrentController()
    }


}
